##################### generated by xml-casa (v2) from importvla.xml #################
##################### b803d2294e3bbe2af64178265f4004b5 ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_importvla import importvla as _importvla_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _importvla:
    """
    importvla ---- Import VLA archive file(s) to a measurement set

    
    Imports an arbitrary number of VLA archive-format data sets into a
    casa measurement set.  If more than one band is present, they will be
    put in the same measurement set but in a separate spectral window.
    The task will handle old style and new style VLA (after July 2007)
    archive data and apply the tsys to the data and to the weights.

    --------- parameter descriptions ---------------------------------------------

    archivefiles  Name of input VLA archive file(s)
                  Default: none.  Must be supplied
                  
                     Examples: 
                     archivefiles = 'AP314_A959519.xp1'
                     archivefiles=['AP314_A950519.xp1',
                     'AP314_A950519.xp2']
    vis           Name of output visibility file
                  Default: none.  Must be supplied
                  
                     Example: vis='NGC7538.ms'
                  
                  NOTE: Will not over-write existing ms of same
                  name. A backup flag-file version 'Original' will
                  be made in vis.flagversions.  See help
                  flagmanager.
    bandname      VLA frequency band name:
                  Default: '' (obtain all bands in the archive
                  file)
                  Options: '4'=48-96 MHz,'P'=298-345
                  MHz,'L'=1.15-1.75 GHz, 'C'=4.2-5.1
                  GHz,'X'=6.8-9.6 GHz,'U'=13.5-16.3 GHz,
                  'K'=20.8-25.8 GHz,'Q'=38-51 GHz
                  
                     Example: bandname='K'
    frequencytol  Tolerance in frequency shift in making spectral windows
                  Default: = 150000.0Hz'
                  
                     Example: frequencytol = 1500000.0 (units = Hz)
                  
                  For Doppler shifted data, less than 10000 Hz may
                  may produce too many unnecessary spectral
                  windows.
    project       Project name to import from archive files
                  Default: '' (all projects in file)
                  
                     Example: project='AL519'             
                     Project = 'al519' or AL519 will work. 
                  
                  WARNING: Do not include leading zeros; project =
                  'AL0519' will not work.
    starttime     Time after which data will be considered for importing
                  Default: '' (all)
                  
                  syntax: starttime = '2003/1/31/05:05:23'. Date
                  must be included!
    stoptime      Time before which data will be considered for
                  importing
                  Default: '' (all)
                  
                  syntax: starttime = '2003/1/31/08:05:23'. Date
                  must be included!
    applytsys     Apply data scaling and weight scaling by nominal
                  sensitivity (~Tsys)
                  Default: True (strongly recommended)
                  Options: True|False
    autocorr      Import autocorrelations to MS
                  Default: False (no autocorrelations)
                  Options: False|True
    antnamescheme 'old' or 'new' antenna names.
                  Default: 'new'
                  Options: new|old
                  
                  * 'new' gives antnenna names 'VA04' or 'EA13 for
                    VLA telescopse 04 and 13 (EVLA)
                  * 'old' gives names '04' or '13'
    keepblanks    Should sources with blank names be filled into the data
                  base?
                  Default: False (do not fill)
                  Options: False|True
                  
                  These scans are tipping scans (as of June 1,
                  2009) and should not be filled in the visibility
                  data set.
    evlabands     Use the EVLA's center frequency and bandwidths for
                  frequencies specified via wavelength or band.
                  Default: False
                  Options: False|True

    --------- examples -----------------------------------------------------------

    
    FOR MORE INFORMATION, SEE THE TASK PAGES OF IMPORTVLA IN CASA DOCS:
    https://casa.nrao.edu/casadocs/
    


    """

    _info_group_ = """import/export"""
    _info_desc_ = """Import VLA archive file(s) to a measurement set"""

    __schema = {'archivefiles': {'type': 'cReqPathVec', 'coerce': [_coerce.to_list,_coerce.expand_pathvec]}, 'vis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'bandname': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ '4', 'X', 'S', 'P', 'K', 'Ka', 'Q', 'U', 'C', 'L', '' ]}, 'frequencytol': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'project': {'type': 'cStr', 'coerce': _coerce.to_str}, 'starttime': {'type': 'cStr', 'coerce': _coerce.to_str}, 'stoptime': {'type': 'cStr', 'coerce': _coerce.to_str}, 'applytsys': {'type': 'cBool'}, 'autocorr': {'type': 'cBool'}, 'antnamescheme': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'old', 'new' ]}, 'keepblanks': {'type': 'cBool'}, 'evlabands': {'type': 'cBool'}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 13 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __project_dflt( self, glb ):
        return ''

    def __project( self, glb ):
        if 'project' in glb: return glb['project']
        return ''

    def __stoptime_dflt( self, glb ):
        return ''

    def __stoptime( self, glb ):
        if 'stoptime' in glb: return glb['stoptime']
        return ''

    def __evlabands_dflt( self, glb ):
        return False

    def __evlabands( self, glb ):
        if 'evlabands' in glb: return glb['evlabands']
        return False

    def __vis_dflt( self, glb ):
        return ''

    def __vis( self, glb ):
        if 'vis' in glb: return glb['vis']
        return ''

    def __archivefiles_dflt( self, glb ):
        return [  ]

    def __archivefiles( self, glb ):
        if 'archivefiles' in glb: return glb['archivefiles']
        return [  ]

    def __antnamescheme_dflt( self, glb ):
        return 'new'

    def __antnamescheme( self, glb ):
        if 'antnamescheme' in glb: return glb['antnamescheme']
        return 'new'

    def __starttime_dflt( self, glb ):
        return ''

    def __starttime( self, glb ):
        if 'starttime' in glb: return glb['starttime']
        return ''

    def __autocorr_dflt( self, glb ):
        return False

    def __autocorr( self, glb ):
        if 'autocorr' in glb: return glb['autocorr']
        return False

    def __keepblanks_dflt( self, glb ):
        return False

    def __keepblanks( self, glb ):
        if 'keepblanks' in glb: return glb['keepblanks']
        return False

    def __bandname_dflt( self, glb ):
        return ''

    def __bandname( self, glb ):
        if 'bandname' in glb: return glb['bandname']
        return ''

    def __applytsys_dflt( self, glb ):
        return True

    def __applytsys( self, glb ):
        if 'applytsys' in glb: return glb['applytsys']
        return True

    def __frequencytol_dflt( self, glb ):
        return '150000.0Hz'

    def __frequencytol( self, glb ):
        if 'frequencytol' in glb: return glb['frequencytol']
        return '150000.0Hz'



    #--------- return inp/go default --------------------------------------------------


    #--------- return subparam values -------------------------------------------------


    #--------- subparam inp output ----------------------------------------------------
    def __archivefiles_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__archivefiles_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return [  ]
        description = 'Name of input VLA archive file(s)'
        value = self.__archivefiles( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'archivefiles': value},{'archivefiles': self.__schema['archivefiles']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('archivefiles',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __vis_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__vis_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Name of output visibility file'
        value = self.__vis( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'vis': value},{'vis': self.__schema['vis']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('vis',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __bandname_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__bandname_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'VLA frequency band name:\'\'=>obtain all bands in the archive file'
        value = self.__bandname( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'bandname': value},{'bandname': self.__schema['bandname']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('bandname',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __frequencytol_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__frequencytol_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return '150000.0Hz'
        description = 'Frequency shift to define a unique spectra window (Hz)'
        value = self.__frequencytol( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'frequencytol': value},{'frequencytol': self.__schema['frequencytol']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('frequencytol',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __project_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__project_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Project name: \'\' => all projects in files'
        value = self.__project( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'project': value},{'project': self.__schema['project']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('project',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __starttime_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__starttime_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Start time to search for data'
        value = self.__starttime( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'starttime': value},{'starttime': self.__schema['starttime']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('starttime',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __stoptime_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__stoptime_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'End time to search for data'
        value = self.__stoptime( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'stoptime': value},{'stoptime': self.__schema['stoptime']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('stoptime',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __applytsys_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__applytsys_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return True
        description = 'Apply nominal sensitivity scaling to data and weights'
        value = self.__applytsys( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'applytsys': value},{'applytsys': self.__schema['applytsys']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('applytsys',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __autocorr_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__autocorr_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Import autocorrelations to MS, if set to True'
        value = self.__autocorr( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'autocorr': value},{'autocorr': self.__schema['autocorr']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('autocorr',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __antnamescheme_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__antnamescheme_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'new'
        description = '\'old\' or \'new\'; \'VA04\' or \'04\' for VLA ant 4'
        value = self.__antnamescheme( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'antnamescheme': value},{'antnamescheme': self.__schema['antnamescheme']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('antnamescheme',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __keepblanks_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__keepblanks_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Fill scans with blank (empty) source names (e.g. tipping scans)'
        value = self.__keepblanks( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'keepblanks': value},{'keepblanks': self.__schema['keepblanks']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('keepblanks',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __evlabands_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__evlabands_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        description = 'Use updated eVLA frequencies and bandwidths for bands and wavelengths'
        value = self.__evlabands( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'evlabands': value},{'evlabands': self.__schema['evlabands']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-13.13s = %s%-23s%s' % ('evlabands',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'archivefiles' in glb: del glb['archivefiles']
        if 'applytsys' in glb: del glb['applytsys']
        if 'project' in glb: del glb['project']
        if 'keepblanks' in glb: del glb['keepblanks']
        if 'evlabands' in glb: del glb['evlabands']
        if 'vis' in glb: del glb['vis']
        if 'bandname' in glb: del glb['bandname']
        if 'frequencytol' in glb: del glb['frequencytol']
        if 'starttime' in glb: del glb['starttime']
        if 'antnamescheme' in glb: del glb['antnamescheme']
        if 'stoptime' in glb: del glb['stoptime']
        if 'autocorr' in glb: del glb['autocorr']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# importvla -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__archivefiles_inp( )
        self.__vis_inp( )
        self.__bandname_inp( )
        self.__frequencytol_inp( )
        self.__project_inp( )
        self.__starttime_inp( )
        self.__stoptime_inp( )
        self.__applytsys_inp( )
        self.__autocorr_inp( )
        self.__antnamescheme_inp( )
        self.__keepblanks_inp( )
        self.__evlabands_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "importvla.last" if os.path.isfile("importvla.last") else "importvla.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('importvla.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['archivefiles'] = self.__archivefiles( self.__globals_( ) )
        _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
        _invocation_parameters['bandname'] = self.__bandname( self.__globals_( ) )
        _invocation_parameters['frequencytol'] = self.__frequencytol( self.__globals_( ) )
        _invocation_parameters['project'] = self.__project( self.__globals_( ) )
        _invocation_parameters['starttime'] = self.__starttime( self.__globals_( ) )
        _invocation_parameters['stoptime'] = self.__stoptime( self.__globals_( ) )
        _invocation_parameters['applytsys'] = self.__applytsys( self.__globals_( ) )
        _invocation_parameters['autocorr'] = self.__autocorr( self.__globals_( ) )
        _invocation_parameters['antnamescheme'] = self.__antnamescheme( self.__globals_( ) )
        _invocation_parameters['keepblanks'] = self.__keepblanks( self.__globals_( ) )
        _invocation_parameters['evlabands'] = self.__evlabands( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-13s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#importvla( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, archivefiles=None, vis=None, bandname=None, frequencytol=None, project=None, starttime=None, stoptime=None, applytsys=None, autocorr=None, antnamescheme=None, keepblanks=None, evlabands=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('importvla.pre')
        _postfile = os.path.realpath('importvla.last')
        task_result = None
        _arguments = [archivefiles,vis,bandname,frequencytol,project,starttime,stoptime,applytsys,autocorr,antnamescheme,keepblanks,evlabands]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if archivefiles is not None: local_global['archivefiles'] = archivefiles
            if vis is not None: local_global['vis'] = vis
            if bandname is not None: local_global['bandname'] = bandname
            if frequencytol is not None: local_global['frequencytol'] = frequencytol
            if project is not None: local_global['project'] = project
            if starttime is not None: local_global['starttime'] = starttime
            if stoptime is not None: local_global['stoptime'] = stoptime
            if applytsys is not None: local_global['applytsys'] = applytsys
            if autocorr is not None: local_global['autocorr'] = autocorr
            if antnamescheme is not None: local_global['antnamescheme'] = antnamescheme
            if keepblanks is not None: local_global['keepblanks'] = keepblanks
            if evlabands is not None: local_global['evlabands'] = evlabands

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['archivefiles'] = self.__archivefiles( local_global )
            _invocation_parameters['vis'] = self.__vis( local_global )
            _invocation_parameters['bandname'] = self.__bandname( local_global )
            _invocation_parameters['frequencytol'] = self.__frequencytol( local_global )
            _invocation_parameters['project'] = self.__project( local_global )
            _invocation_parameters['starttime'] = self.__starttime( local_global )
            _invocation_parameters['stoptime'] = self.__stoptime( local_global )
            _invocation_parameters['applytsys'] = self.__applytsys( local_global )
            _invocation_parameters['autocorr'] = self.__autocorr( local_global )
            _invocation_parameters['antnamescheme'] = self.__antnamescheme( local_global )
            _invocation_parameters['keepblanks'] = self.__keepblanks( local_global )
            _invocation_parameters['evlabands'] = self.__evlabands( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            

        else:
            # invoke with inp/go semantics
            _invocation_parameters['archivefiles'] = self.__archivefiles( self.__globals_( ) )
            _invocation_parameters['vis'] = self.__vis( self.__globals_( ) )
            _invocation_parameters['bandname'] = self.__bandname( self.__globals_( ) )
            _invocation_parameters['frequencytol'] = self.__frequencytol( self.__globals_( ) )
            _invocation_parameters['project'] = self.__project( self.__globals_( ) )
            _invocation_parameters['starttime'] = self.__starttime( self.__globals_( ) )
            _invocation_parameters['stoptime'] = self.__stoptime( self.__globals_( ) )
            _invocation_parameters['applytsys'] = self.__applytsys( self.__globals_( ) )
            _invocation_parameters['autocorr'] = self.__autocorr( self.__globals_( ) )
            _invocation_parameters['antnamescheme'] = self.__antnamescheme( self.__globals_( ) )
            _invocation_parameters['keepblanks'] = self.__keepblanks( self.__globals_( ) )
            _invocation_parameters['evlabands'] = self.__evlabands( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-13s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#importvla( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'importvla', [ 'archivefiles=' + repr(_pc.document['archivefiles']), 'vis=' + repr(_pc.document['vis']), 'bandname=' + repr(_pc.document['bandname']), 'frequencytol=' + repr(_pc.document['frequencytol']), 'project=' + repr(_pc.document['project']), 'starttime=' + repr(_pc.document['starttime']), 'stoptime=' + repr(_pc.document['stoptime']), 'applytsys=' + repr(_pc.document['applytsys']), 'autocorr=' + repr(_pc.document['autocorr']), 'antnamescheme=' + repr(_pc.document['antnamescheme']), 'keepblanks=' + repr(_pc.document['keepblanks']), 'evlabands=' + repr(_pc.document['evlabands']) ] )
            task_result = _importvla_t( _pc.document['archivefiles'],_pc.document['vis'],_pc.document['bandname'],_pc.document['frequencytol'],_pc.document['project'],_pc.document['starttime'],_pc.document['stoptime'],_pc.document['applytsys'],_pc.document['autocorr'],_pc.document['antnamescheme'],_pc.document['keepblanks'],_pc.document['evlabands'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('importvla')
            casalog.post("Exception Reported: Error in importvla: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            task_result = False
        try:
            os.rename(_prefile,_postfile)
        except:
            pass
        finally:
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'importvla', task_result )
        return task_result

importvla = _importvla( )

