##################### generated by xml-casa (v2) from specsmooth.xml ################
##################### 750dccf3562318cbdb15c36de3abf077 ##############################
from __future__ import absolute_import
from casashell.private.stack_manip import find_local as __sf__
from casashell.private.stack_manip import find_frame as _find_frame
from casatools.typecheck import validator as _pc
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from casatasks.private.task_specsmooth import specsmooth as _specsmooth_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from collections import OrderedDict
import numpy
import sys
import os

import shutil

def static_var(varname, value):
    def decorate(func):
        setattr(func, varname, value)
        return func
    return decorate

class _specsmooth:
    """
    specsmooth ---- Smooth an image region in one dimension

    --------- parameter descriptions ---------------------------------------------

    imagename Name of the input image
    outfile   Output image name.
    box       Rectangular region to select in direction plane. Default is to use the entire direction plane.
    chans     Channels to use. Channels must be contiguous. Default is to use all channels.
    stokes    Stokes planes to use. Planes specified must be contiguous. Default is to use all Stokes planes.
    region    Region selection. Default is to use the full image.
    mask      Mask to use. Default is none..
    overwrite Overwrite the output if it exists?
    stretch   Stretch the mask if necessary and possible? Default False
    axis      The profile axis. Default: use the spectral axis if one exists, axis 0 otherwise (<0).
    function  Convolution function. hanning and boxcar are supported functions. Minimum match is supported.
    width     Width of boxcar, in pixels.
    dmethod   Decimation method. "" means no decimation, "copy" and "mean" are also supported (minimum match).
    [1;42mRETURNS[1;m      record

    --------- examples -----------------------------------------------------------

    
    
    Smooth an image region in one dimension.
    
    ARAMETER SUMMARY
    imagename       Name of the input (CASA, FITS, MIRIAD) image
    box             Rectangular region to select in direction plane.
    Default is to use the entire direction plane. Only a single box may be specified.
    chans           Channels to use. Channels must be contiguous.
    Default is to use all channels.
    stokes          Stokes planes to use. Planes specified must be
    contiguous. Default is to use all Stokes planes.
    region          Region selection. Default is to use the full image.
    mask            Mask to use. Default is none.
    overwrite       If the specified outfile already exists, overwrite it if True.
    stretch         Stretch the input mask if necessary and possible? Only used if a mask is specified.
    
    axis            Pixel axis along which to do the convolution <0 means use the spectral axis.
    function        Convolution function to use. Supported values are "boxcar" and "hanning". Minimum
    match is supported.
    width           Width of boxcar in pixels. Used only if function parameter minimally matches "boxcar".
    dmethod         Plane decimation method. "" means no decimation should be performed. Other supported
    values are "copy" and "mean". Minimal match is supported. See below for details.
    
    This application performs one dimensional convolution along a specified axis of an image
    or selected region of an image. Hanning smoothing and boxcar smoothing are supported.
    Both float valued and complex valued images are supported. Masked pixel values are set to
    zero prior to convolution. All nondefault pixel masks are ignored during
    the calculation. The convolution is done in the image domain (i.e., not
    with an FFT).
    
    BOXCAR SMOOTHING
    
    One dimensional boxcar convolution is defined by
    
    z[i] = (y[i] + y[i+i] + ... + y[i+w])/w
    
    where z[i] is the value at pixel i in the box car smoothed image, y[k]
    is the pixel value of the input image at pixel k, and w is a postivie integer
    representing the width of the boxcar in pixels.  The length of the axis along which the
    convolution is to occur must be at least w pixels in the selected region,
    unless decimation using the mean function is chosen in which case the axis
    length must be at least 2*w (see below).
    
    If dmethod="" (no decimation), the length of the output axis will be equal
    to the length of the input axis - w + 1. The pixel mask, ORed with the OTF mask
    if specified, is copied from the selected region of the input image to the
    output image. Thus for example, if the selected region in the input image has
    six planes along the convolution axis, if the specified boxcar width is 2,
    and if the pixel values, which are all unmasked, on a slice along this axis
    are [1, 2, 5, 10, 17, 26], then the corresponding output slice will be of
    length five pixels and the output pixel values will be [1.5, 3.5, 7.5, 13.5, 21.5].
    
    If dmethod="copy", the output image is the image calculated
    if dmethod="", except that only every wth plane is kept. Both the pixel and mask
    values of these planes are copied directly to the output image, without further
    processing. Thus for example, if the selected region in the input image has six
    planes along the convolution axis, the boxcar width is chosen to be 2, and if
    the pixel values, which are all unmasked, on a slice along this axis are [1, 2,
    5, 10, 17, 26], the corresponding output pixel values will be [1.5, 7.5, 21.5].
    
    If dmethod="mean", first the image described in the dmethod=""
    case is calculated. Then, the ith plane of the output image is calculated by
    averaging the i*w to the (i+1)*w-1  planes of this intermediate image. Thus, for
    example, if the selected region in the input image has six planes along the
    convolution axis, the boxcar width is chosen to be 2, and if the pixel values,
    which are all unmasked, on a slice along this axis are [1, 2, 5, 10, 17, 26],
    then the corresponding output pixel values will be [2.5, 10.5]. Any pixels at the
    end of the convolution axis of the intermediate image that do not fall into a complete bin of
    width w are ignored. Masked values are taken into consideration when forming this
    average, so if one of the values is masked, it is not used in the average. If at
    least one of the values in the intermediate image bin is not masked, the
    corresponding output pixel will not be masked.
    
    HANNING SMOOTHING
    
    Hanning convolution of one axis of an image is defined by
    
    z[i] = 0.25*y[i-1] + 0.5*y[i] + 0.25*y[i+1]       (equation 1)
    
    where z[i] is the value at pixel i in the hanning smoothed image, and
    y[i-1], y[i], and y[i+1] are the values of the input image at pixels i-1,
    i, and i+1 respectively. The length of the axis along which the convolution is
    to occur must be at least three pixels in the selected region.
    
    If dmethod="" (no decimation of image planes), the length of the output axis will
    be the same as that of the input axis. The output pixel values along the convolution
    axis will be related to those of the input values according to equation 1, except
    the first and last pixels. In that case,
    
    z[0] = 0.5*(y[0] + y[1])
    
    and,
    
    z[N-1] = 0.5*(y[N-2] + y[N-1])
    
    where N is the number of pixels along the convolution aixs.
    The pixel mask, ORed with the OTF mask if specified, is copied from the selected
    region of the input image to the output image. Thus for example, if the selected
    region in the input image has six planes along the convolution axis, and if the pixel
    values, which are all unmasked, on a slice along this axis are [1, 2, 5, 10, 17, 26],
    the corresponding output pixel values will be [1.5, 2.5, 5.5, 10.5, 17.5, 21.5].
    
    If dmethod="copy", the output image is the image calculated if
    dmethod="", except that only the odd-numbered planes are kept. Furthermore, if the
    number of planes along the convolution axis in the selected region of the input image
    is even, the last odd number plane is also discarded. Thus, if the selected region
    has N pixels along the convolution axis in the input image, along the convolution
    axis the output image will have (N-1)/2 planes if N is odd, or (N-2)/2 planes if N
    is even. The pixel and mask values are copied directly, without further
    processing. Thus for example, if the selected region in the input image has six planes
    along the convolution axis, and if the pixel values, which are all unmasked, on a slice
    along this axis are [1, 2, 5, 10, 17, 26], the corresponding output pixel values will be
    [2.5, 10.5].
    
    If dmethod="mean", first the image described in the dmethod="" case
    is calculated. The first plane and last plane(s) of that image are then discarded as
    described in the dmethod="copy" case. Then, the ith plane of the output
    image is calculated by averaging the (2*i)th and (2*i + 1)th planes of the intermediate
    image. Thus for example, if the selected region in the input image has six planes
    along the convolution axis, and if the pixel values, which are all unmasked, on a slice
    along this axis are [1, 2, 5, 10, 17, 26], the corresponding output pixel values will be
    [4.0, 14.0]. Masked values are taken into consideration when forming this average, so if
    one of the values is masked, it is not used in the average. If at least one of the values
    in the input pair is not masked, the corresponding output pixel will not be masked.
    
    
    EXAMPLES
    
    # boxcar smooth the spectral axis by 3 pixels, say it's axis 2 and only
    # write every other pixel
    specsmooth(imagename="mynonsmoothed.im", outfile="myboxcarsmoothed.im",
    axis=2, function="boxcar", dmethod="copy", width=3, overwrite=True)
    
    # hanning smooth the spectral axis, say it's axis 2 and do not perform decimation
    # of image planes
    specsmooth(imagename="mynonsmoothed.im", outfile="myhanningsmoothed.im",
    axis=2, dmethod=""," overwrite=True)


    """

    _info_group_ = """analysis"""
    _info_desc_ = """Smooth an image region in one dimension"""

    __schema = {'imagename': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'outfile': {'type': 'cStr', 'coerce': _coerce.to_str}, 'box': {'type': 'cStr', 'coerce': _coerce.to_str}, 'chans': {'type': 'cStr', 'coerce': _coerce.to_str}, 'stokes': {'type': 'cStr', 'coerce': _coerce.to_str}, 'region': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'mask': {'type': 'cStr', 'coerce': _coerce.to_str}, 'overwrite': {'type': 'cBool'}, 'stretch': {'type': 'cBool'}, 'axis': {'type': 'cInt'}, 'function': {'type': 'cStr', 'coerce': _coerce.to_str}, 'width': {'type': 'cInt'}, 'dmethod': {'type': 'cStr', 'coerce': _coerce.to_str}}

    def __init__(self):
        self.__stdout = None
        self.__stderr = None
        self.__root_frame_ = None

    def __globals_(self):
        if self.__root_frame_ is None:
            self.__root_frame_ = _find_frame( )
            assert self.__root_frame_ is not None, "could not find CASAshell global frame"
        return self.__root_frame_

    def __to_string_(self,value):
        if type(value) is str:
            return "'%s'" % value
        else:
            return str(value)

    def __validate_(self,doc,schema):
        return _pc.validate(doc,schema)

    def __do_inp_output(self,param_prefix,description_str,formatting_chars):
        out = self.__stdout or sys.stdout
        description = description_str.split( )
        prefix_width = 23 + 12 + 4
        output = [ ]
        addon = ''
        first_addon = True
        if len(description) == 0:
            out.write(param_prefix + " #\n")
            return
        while len(description) > 0:
            ## starting a new line.....................................................................
            if len(output) == 0:
                ## for first line add parameter information............................................
                if len(param_prefix)-formatting_chars > prefix_width - 1:
                    output.append(param_prefix)
                    continue
                addon = param_prefix + ' #'
                first_addon = True
                addon_formatting = formatting_chars
            else:
                ## for subsequent lines space over prefix width........................................
                addon = (' ' * prefix_width) + '#'
                first_addon = False
                addon_formatting = 0
            ## if first word of description puts us over the screen width, bail........................
            if len(addon + description[0]) - addon_formatting + 1 > self.term_width:
                ## if we're doing the first line make sure it's output.................................
                if first_addon: output.append(addon)
                break
            while len(description) > 0:
                ## if the next description word puts us over break for the next line...................
                if len(addon + description[0]) - addon_formatting + 1 > self.term_width: break
                addon = addon + ' ' + description[0]
                description.pop(0)
            output.append(addon)
        out.write('\n'.join(output) + '\n')

    #--------- return nonsubparam values ----------------------------------------------

    def __outfile_dflt( self, glb ):
        return ''

    def __outfile( self, glb ):
        if 'outfile' in glb: return glb['outfile']
        return ''

    def __dmethod_dflt( self, glb ):
        return 'copy'

    def __dmethod( self, glb ):
        if 'dmethod' in glb: return glb['dmethod']
        return 'copy'

    def __axis_dflt( self, glb ):
        return int(-1)

    def __axis( self, glb ):
        if 'axis' in glb: return glb['axis']
        return int(-1)

    def __mask_dflt( self, glb ):
        return ''

    def __mask( self, glb ):
        if 'mask' in glb: return glb['mask']
        return ''

    def __stokes_dflt( self, glb ):
        return ''

    def __stokes( self, glb ):
        if 'stokes' in glb: return glb['stokes']
        return ''

    def __region_dflt( self, glb ):
        return ''

    def __region( self, glb ):
        if 'region' in glb: return glb['region']
        return ''

    def __function_dflt( self, glb ):
        return 'hanning'

    def __function( self, glb ):
        if 'function' in glb: return glb['function']
        return 'hanning'

    def __chans_dflt( self, glb ):
        return ''

    def __chans( self, glb ):
        if 'chans' in glb: return glb['chans']
        return ''

    def __imagename_dflt( self, glb ):
        return ''

    def __imagename( self, glb ):
        if 'imagename' in glb: return glb['imagename']
        return ''

    def __box_dflt( self, glb ):
        return ''

    def __box( self, glb ):
        if 'box' in glb: return glb['box']
        return ''

    #--------- return non subparam/when values ---------------------------------------------
    def __width( self, glb ):
        if 'width' in glb: return glb['width']
        return int(2)

    #--------- return inp/go default --------------------------------------------------
    def __stretch_dflt( self, glb ):
        if self.__mask( glb ) != "": return bool(False)
        return None
    def __overwrite_dflt( self, glb ):
        if self.__outfile( glb ) != "": return bool(False)
        return None
    def __width_dflt( self, glb ):
        if self.__function( glb ) != "hanning": return int(2)
        return None

    #--------- return subparam values -------------------------------------------------
    def __overwrite( self, glb ):
        if 'overwrite' in glb: return glb['overwrite']
        dflt = self.__overwrite_dflt( glb )
        if dflt is not None: return dflt
        return False
    def __stretch( self, glb ):
        if 'stretch' in glb: return glb['stretch']
        dflt = self.__stretch_dflt( glb )
        if dflt is not None: return dflt
        return False

    #--------- subparam inp output ----------------------------------------------------
    def __imagename_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__imagename_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Name of the input image'
        value = self.__imagename( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'imagename': value},{'imagename': self.__schema['imagename']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('imagename',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __outfile_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__outfile_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Output image name.'
        value = self.__outfile( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'outfile': value},{'outfile': self.__schema['outfile']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-12.12s =\x1B[0m %s%-23s%s' % ('outfile',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __box_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__box_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Rectangular region to select in direction plane. Default is to use the entire direction plane.'
        value = self.__box( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'box': value},{'box': self.__schema['box']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('box',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __chans_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__chans_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Channels to use. Channels must be contiguous. Default is to use all channels.'
        value = self.__chans( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'chans': value},{'chans': self.__schema['chans']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('chans',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __stokes_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__stokes_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Stokes planes to use. Planes specified must be contiguous. Default is to use all Stokes planes.'
        value = self.__stokes( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'stokes': value},{'stokes': self.__schema['stokes']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('stokes',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __region_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__region_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Region selection. Default is to use the full image.'
        value = self.__region( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'region': value},{'region': self.__schema['region']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('region',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __mask_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__mask_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return ''
        description = 'Mask to use. Default is none..'
        value = self.__mask( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'mask': value},{'mask': self.__schema['mask']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-12.12s =\x1B[0m %s%-23s%s' % ('mask',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __overwrite_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__overwrite_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        if self.__overwrite_dflt( self.__globals_( ) ) is not None:
             description = 'Overwrite the output if it exists?'
             value = self.__overwrite( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'overwrite': value},{'overwrite': self.__schema['overwrite']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-9.9s =\x1B[0m %s%-23s%s' % ('overwrite',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __stretch_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__stretch_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return False
        if self.__stretch_dflt( self.__globals_( ) ) is not None:
             description = 'Stretch the mask if necessary and possible? Default False'
             value = self.__stretch( self.__globals_( ) )
             (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'stretch': value},{'stretch': self.__schema['stretch']}) else ('\x1B[91m','\x1B[0m')
             self.__do_inp_output('   \x1B[92m%-9.9s =\x1B[0m %s%-23s%s' % ('stretch',pre,self.__to_string_(value),post),description,9+len(pre)+len(post))
    def __axis_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__axis_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return int(-1)
        description = 'The profile axis. Default: use the spectral axis if one exists, axis 0 otherwise (<0).'
        value = self.__axis( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'axis': value},{'axis': self.__schema['axis']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('axis',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __function_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__function_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'hanning'
        description = 'Convolution function. hanning and boxcar are supported functions. Minimum match is supported.'
        value = self.__function( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'function': value},{'function': self.__schema['function']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('\x1B[1m\x1B[47m%-12.12s =\x1B[0m %s%-23s%s' % ('function',pre,self.__to_string_(value),post),description,13+len(pre)+len(post))
    def __width_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__width_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return int(2)
        description = 'Width of boxcar, in pixels.'
        value = self.__width( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'width': value},{'width': self.__schema['width']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('width',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))
    def __dmethod_inp(self):
        def xml_default( ):
            ## play the crazy subparameter shell game
            dflt = self.__dmethod_dflt( self.__globals_( ) )
            if dflt is not None: return dflt
            return 'copy'
        description = 'Decimation method. "" means no decimation, "copy" and "mean" are also supported (minimum match).'
        value = self.__dmethod( self.__globals_( ) )
        (pre,post) = (('','') if value == xml_default( ) else ('\x1B[34m','\x1B[0m')) if self.__validate_({'dmethod': value},{'dmethod': self.__schema['dmethod']}) else ('\x1B[91m','\x1B[0m')
        self.__do_inp_output('%-12.12s = %s%-23s%s' % ('dmethod',pre,self.__to_string_(value),post),description,0+len(pre)+len(post))

    #--------- global default implementation-------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def set_global_defaults(self):
        self.set_global_defaults.state['last'] = self
        glb = self.__globals_( )
        if 'stokes' in glb: del glb['stokes']
        if 'outfile' in glb: del glb['outfile']
        if 'mask' in glb: del glb['mask']
        if 'dmethod' in glb: del glb['dmethod']
        if 'stretch' in glb: del glb['stretch']
        if 'imagename' in glb: del glb['imagename']
        if 'function' in glb: del glb['function']
        if 'chans' in glb: del glb['chans']
        if 'region' in glb: del glb['region']
        if 'box' in glb: del glb['box']
        if 'axis' in glb: del glb['axis']
        if 'overwrite' in glb: del glb['overwrite']
        if 'width' in glb: del glb['width']


    #--------- inp function -----------------------------------------------------------
    def inp(self):
        print("# specsmooth -- %s" % self._info_desc_)
        self.term_width, self.term_height = shutil.get_terminal_size(fallback=(80, 24))
        self.__imagename_inp( )
        self.__outfile_inp( )
        self.__box_inp( )
        self.__chans_inp( )
        self.__stokes_inp( )
        self.__region_inp( )
        self.__mask_inp( )
        self.__overwrite_inp( )
        self.__stretch_inp( )
        self.__axis_inp( )
        self.__function_inp( )
        self.__width_inp( )
        self.__dmethod_inp( )

    #--------- tget function ----------------------------------------------------------
    @static_var('state', __sf__('casa_inp_go_state'))
    def tget(self,savefile=None):
        from casashell.private.stack_manip import find_frame
        from runpy import run_path
        filename = savefile
        if filename is None:
            filename = "specsmooth.last" if os.path.isfile("specsmooth.last") else "specsmooth.saved"
        if os.path.isfile(filename):
            glob = find_frame( )
            newglob = run_path( filename, init_globals={ } )
            for i in newglob:
                glob[i] = newglob[i]
            self.tget.state['last'] = self
        else:
            print("could not find last file: %s\nsetting defaults instead..." % filename)
            self.set_global_defaults( )

    #--------- tput function ----------------------------------------------------------
    def tput(self,outfile=None):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s

        _postfile = outfile if outfile is not None else os.path.realpath('specsmooth.last')

        _invocation_parameters = OrderedDict( )
        _invocation_parameters['imagename'] = self.__imagename( self.__globals_( ) )
        _invocation_parameters['outfile'] = self.__outfile( self.__globals_( ) )
        _invocation_parameters['box'] = self.__box( self.__globals_( ) )
        _invocation_parameters['chans'] = self.__chans( self.__globals_( ) )
        _invocation_parameters['stokes'] = self.__stokes( self.__globals_( ) )
        _invocation_parameters['region'] = self.__region( self.__globals_( ) )
        _invocation_parameters['mask'] = self.__mask( self.__globals_( ) )
        _invocation_parameters['overwrite'] = self.__overwrite( self.__globals_( ) )
        _invocation_parameters['stretch'] = self.__stretch( self.__globals_( ) )
        _invocation_parameters['axis'] = self.__axis( self.__globals_( ) )
        _invocation_parameters['function'] = self.__function( self.__globals_( ) )
        _invocation_parameters['width'] = self.__width( self.__globals_( ) )
        _invocation_parameters['dmethod'] = self.__dmethod( self.__globals_( ) )

        try:
            with open(_postfile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-9s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#specsmooth( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: return False
        return True

    def __call__( self, imagename=None, outfile=None, box=None, chans=None, stokes=None, region=None, mask=None, overwrite=None, stretch=None, axis=None, function=None, width=None, dmethod=None ):
        def noobj(s):
           if s.startswith('<') and s.endswith('>'):
               return "None"
           else:
               return s
        _prefile = os.path.realpath('specsmooth.pre')
        _postfile = os.path.realpath('specsmooth.last')
        task_result = None
        _arguments = [imagename,outfile,box,chans,stokes,region,mask,overwrite,stretch,axis,function,width,dmethod]
        _invocation_parameters = OrderedDict( )
        if any(map(lambda x: x is not None,_arguments)):
            # invoke python style
            # set the non sub-parameters that are not None
            local_global = { }
            if imagename is not None: local_global['imagename'] = imagename
            if outfile is not None: local_global['outfile'] = outfile
            if box is not None: local_global['box'] = box
            if chans is not None: local_global['chans'] = chans
            if stokes is not None: local_global['stokes'] = stokes
            if region is not None: local_global['region'] = region
            if mask is not None: local_global['mask'] = mask
            if axis is not None: local_global['axis'] = axis
            if function is not None: local_global['function'] = function
            if width is not None: local_global['width'] = width
            if dmethod is not None: local_global['dmethod'] = dmethod

            # the invocation parameters for the non-subparameters can now be set - this picks up those defaults
            _invocation_parameters['imagename'] = self.__imagename( local_global )
            _invocation_parameters['outfile'] = self.__outfile( local_global )
            _invocation_parameters['box'] = self.__box( local_global )
            _invocation_parameters['chans'] = self.__chans( local_global )
            _invocation_parameters['stokes'] = self.__stokes( local_global )
            _invocation_parameters['region'] = self.__region( local_global )
            _invocation_parameters['mask'] = self.__mask( local_global )
            _invocation_parameters['axis'] = self.__axis( local_global )
            _invocation_parameters['function'] = self.__function( local_global )
            _invocation_parameters['width'] = self.__width( local_global )
            _invocation_parameters['dmethod'] = self.__dmethod( local_global )

            # the sub-parameters can then be set. Use the supplied value if not None, else the function, which gets the appropriate default
            _invocation_parameters['overwrite'] = self.__overwrite( _invocation_parameters ) if overwrite is None else overwrite
            _invocation_parameters['stretch'] = self.__stretch( _invocation_parameters ) if stretch is None else stretch

        else:
            # invoke with inp/go semantics
            _invocation_parameters['imagename'] = self.__imagename( self.__globals_( ) )
            _invocation_parameters['outfile'] = self.__outfile( self.__globals_( ) )
            _invocation_parameters['box'] = self.__box( self.__globals_( ) )
            _invocation_parameters['chans'] = self.__chans( self.__globals_( ) )
            _invocation_parameters['stokes'] = self.__stokes( self.__globals_( ) )
            _invocation_parameters['region'] = self.__region( self.__globals_( ) )
            _invocation_parameters['mask'] = self.__mask( self.__globals_( ) )
            _invocation_parameters['overwrite'] = self.__overwrite( self.__globals_( ) )
            _invocation_parameters['stretch'] = self.__stretch( self.__globals_( ) )
            _invocation_parameters['axis'] = self.__axis( self.__globals_( ) )
            _invocation_parameters['function'] = self.__function( self.__globals_( ) )
            _invocation_parameters['width'] = self.__width( self.__globals_( ) )
            _invocation_parameters['dmethod'] = self.__dmethod( self.__globals_( ) )
        try:
            with open(_prefile,'w') as _f:
                for _i in _invocation_parameters:
                    _f.write("%-9s = %s\n" % (_i,noobj(repr(_invocation_parameters[_i]))))
                _f.write("#specsmooth( ")
                count = 0
                for _i in _invocation_parameters:
                    _f.write("%s=%s" % (_i,noobj(repr(_invocation_parameters[_i]))))
                    count += 1
                    if count < len(_invocation_parameters): _f.write(",")
                _f.write(" )\n")
        except: pass
        try:
            _logging_state_ = None
            assert _pc.validate(_invocation_parameters,self.__schema), create_error_string(_pc.errors)
            _logging_state_ = _start_log( 'specsmooth', [ 'imagename=' + repr(_pc.document['imagename']), 'outfile=' + repr(_pc.document['outfile']), 'box=' + repr(_pc.document['box']), 'chans=' + repr(_pc.document['chans']), 'stokes=' + repr(_pc.document['stokes']), 'region=' + repr(_pc.document['region']), 'mask=' + repr(_pc.document['mask']), 'overwrite=' + repr(_pc.document['overwrite']), 'stretch=' + repr(_pc.document['stretch']), 'axis=' + repr(_pc.document['axis']), 'function=' + repr(_pc.document['function']), 'width=' + repr(_pc.document['width']), 'dmethod=' + repr(_pc.document['dmethod']) ] )
            task_result = _specsmooth_t( _pc.document['imagename'],_pc.document['outfile'],_pc.document['box'],_pc.document['chans'],_pc.document['stokes'],_pc.document['region'],_pc.document['mask'],_pc.document['overwrite'],_pc.document['stretch'],_pc.document['axis'],_pc.document['function'],_pc.document['width'],_pc.document['dmethod'] )
        except Exception as e:
            from traceback import format_exc
            from casatasks import casalog
            casalog.origin('specsmooth')
            casalog.post("Exception Reported: Error in specsmooth: %s" % str(e),'SEVERE')
            casalog.post(format_exc( ))
            task_result = False
        try:
            os.rename(_prefile,_postfile)
        except:
            pass
        finally:
            if _logging_state_:
                task_result = _end_log( _logging_state_, 'specsmooth', task_result )
        return task_result

specsmooth = _specsmooth( )

